;################################################################### CPDRV.INC
; Purpose.....: Driver for Christiani printer which came with the
;		Mikroprozessor Labor. This is an include file to be used
;		by applications which require the printer and do not have the
;		Disassembler/Hexdump ROM available.
;
; Target......: MPF-1
; Assembler...: ZMAC V18oct2022
;
;======================================================================= notes
; Requires....: The application using this driver has to export some 
;		variables to avoid the use of fixed RAM space:
;		vCPFLG: flags address
;		iCPFUK: user key pressed bit number in flags
;		vCPPBF: print buffer address
;		vCPPAT:	a word variable for storing I/O ports table address
;
; Exports.....: CPIFS(): Search for connected printer at EPB-PPI or MPF-PIO.
;		CPCB() : Clear printer buffer, fill with spaces.
;		CPPR() : Print a line
;		CPPT() : Print zero-terminated text
;		CPLF() : Print a line feed 
;		rCPNP  : Display pattern for '    nP' (no printer) message
;
;===================================================================== history
; 070824-V1.00: Extracted from Disassembler into this include file so it can
;		be used from other applications as well.
;#############################################################################

;***************************************************************** definitions
bCPPBZ		EQU	24		; printer characters per line
bCPPPI		EQU	1		; PPI on EPB-MPF
bCPPIO		EQU	2               ; PIO on MPF

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ I/O ports
;============================================================= system PPI, U14
pCPSYA		EQU	00H		; sys PPI, port A
iCPSYU		EQU	6		; user key bit

;============================================================= EPB-MPF PPI, U1
pCPPIC          EQU     0CFH            ; PPI control
pCPPID          EQU     0CCH            ;     port A, data,   PA0-PA5
pCPPIM          EQU     0CDH            ;     port B, motor,  PB0
pCPPIS          EQU     0CEH            ;     port C, status, PC3

bCPIVM		EQU	01H		; value for motor on/off bit
bCPIVS		EQU	08H		; value for status bit
		
;============================================================ Z80-PIO on MPF-1
pCPPPC		EQU	083H		; PIO control
pCPPPD		EQU	081H		;     port B, data,   PB0-PB5
pCPPPM		EQU	081H		;     port B, motor,  PB6
pCPPPS		EQU	081H		;     port B, status, PB7

bCPPVM		EQU	40H		; value for motor on/off bit
bCPPVS		EQU	80H		; value for status bit

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ implementation
;===================================================================== CPIFS()
; Purpose.....: Check on which interface the printer is connected, then
;		setup this port.
; Input.......: CY: 1= initialize
;		    0= de-initialize, set all ports as input
; Output......: CY: 1= printer not found
;		    0= printer found
; Registers...: AF destroyed
; Labels......: C)hristiani P)rinter I)nterF)ace S)etup
;=============================================================================
CPIFS:          PUSH	HL

		JR	NC,CPIFIR	; CY=0, deinitialize

;-------------------------------------------------------------- PPI on EPB-MPF		
		LD	A,10011011B	; PA=in, PB=in, PCL=in, PCH=in
		OUT	(pCPPIC),A	; write to EPB-PPI control port
		IN	A,(pCPPIS) 	; get status
		CP	0F8H		; b3=status, b4-7 pulled up
		JR	NZ,CPIFSP	; not this pattern, try PIO

		LD	A,10001001B	; PA=out, PB=out, PCL=in, PCH=in
		OUT	(pCPPIC),A	; write to EPB-PPI control port
		LD	HL,cCPIFI	; HL = PPI ports table address
		LD	(vCPPAT),HL     ; save it
		XOR	A		; CY=0, printer found
		LD	A,bCPPPI	; return PPI interface
		JR	CPIFSX		; done,exit

;-------------------------------------------------------------- Z80 PIO on MPF
CPIFSP:		LD	A,01001111B	; mode 1, input
		OUT	(pCPPPC),A	; write to PIO-B control port
		IN	A,(pCPPPS) 	; get status
		AND	080H		; b7=status, b0-6 floating
		LD	A,0		; no interface
		SCF			; preset to printer not found
		JR	Z,CPIFSX	; status not set, printer not found

		LD	A,11001111B	; mode 3, control
		OUT	(pCPPPC),A	; write to PIO-B control port
		XOR	A
		OUT	(pCPPPD),A	; preset all bits to zero
	        LD	A,10000000B	; PB7 input, PB6-0 outputs
		OUT	(pCPPPC),A	; write to PIO-B control port
		LD	HL,cCPIFP	; HL = PIO ports table address
		LD	(vCPPAT),HL     ; save it
		XOR	A		; CY=0, printer found
		LD	A,bCPPIO	; return PIO interface
		JR	CPIFSX		; done,exit

;------------------------------------------------------- deinit interface chip
CPIFIR:		LD	HL,(vCPPAT)	; HL= ports table address
		LD	A,L		; check if initialized
		OR	H
		JR	Z,CPIFSX        ; no initialized, exit

		LD	HL,0            ; set interface to de-initialized
		LD	(vCPPAT),HL
		
		CP	LOW cCPIFI	; is it PPI?
		JR	NZ,CPIFPR	; no, must be PIO		
		LD	A,10011011B	; PA=in, PB=in, PCL=in, PCH=in
		OUT	(pCPPIC),A	; write to EPB-PPI control port
		JR	CPIFSX		; done, exit
				
CPIFPR:		LD	A,01001111B	; de-init: mode 1, input
		OUT	(pCPPPC),A	; write to PIO-A control port

CPIFSX:		POP	HL
		RET

;--------------------------------------------------- ports and port bits table
bCPPOD		EQU	0		; offset data port
bCPPOM		EQU	1		;        motor port (value+1)
bCPPOS		EQU	3		;        status port (value+1)

cCPIFI:	        DB	pCPPID		; PPI data port (PA)
	        DB	pCPPIM		;     motor port (PB)
	        DB	bCPIVM		;     value for motor on/off
		DB	pCPPIS		;     status port (PC)
		DB	bCPIVS		;     value for status

cCPIFP:	        DB	pCPPPD		; PIO data port (PB)
	        DB	pCPPPM		;     motor port (PB)
	        DB	bCPPVM		;     value for motor on/off
		DB	pCPPPS		;     status port (PB)
		DB	bCPPVS		;     value for status

;------------------------------------ display pattern for 'no printer' message
rCPNP:		DB	01FH,023H,000H	; '   nP'
		DB	000H,000H,000H
;================================================================= eoF CPIFS()

;===================================================================== CPGUK()
; Purpose.....: Check status of user key
; Output......: Z: 1= user key was pressed
;		   0= it was not
; Registers...: AF destroyed
; Labels......: C)hristiani P)printer G)et U)ser K)ey
;=============================================================================
CPGUK:		IN	A,(pCPSYA)	; ex240627-1: read system PPI port A
		BIT	iCPSYU,A	; is user key pressed (active-low)?
      		JR	NZ,CPGUKX	; no, return with Z=0

		LD	A,(vCPFLG)	; yes, set user key pressed flag
		SET	iCPFUK,A
		LD	(vCPFLG),A
CPGUKX:		RET
;================================================================= eoF CPGUK()
        	
;====================================================================== CPCB()
; Purpose.....: Clear print buffer, fill with spaces
; Registers...: All saved
; Labels......: C)hristiani P)inter C)lear B)uffer
;=============================================================================
CPCB:		PUSH	HL
		PUSH	BC
		
		LD	HL,vCPPBF	; HL= line buffer
		LD	B,bCPPBZ	; B= buffer size
CPCBL:		LD	(HL),020H	; init buffer
		INC	HL
		DJNZ	CPCBL		; until whole buffer done	

		POP	BC
		POP	HL
		RET
;================================================================== eoF CPCL()

;===================================================================== CPCNV()
; Purpose.....: Convert characters to Christiani printer character set
; Input.......: HL: points to character to check/convert
; Output......: A:  converted character 
; Registers...: AF destroyed 
;=============================================================================
CPCNV:		PUSH	BC
		PUSH	IX
		
		LD	B,5		; B= max number of conversions
		LD	A,(HL)		; A= character to check/convert
		LD	IX,tCPCNV	; IX= conversion table
		
CPCNVL:		CP	(IX+0)
		JR	Z,CPCNVD
		INC	IX
		INC	IX
		DJNZ	CPCNVL		; check all conversions
		JR	CPCNVX		; not found, exit						
		
CPCNVD:		LD	A,(IX+1)	; A= replacement character				
    		
CPCNVX:		POP	IX
		POP	BC
		RET

;------------------------------------------------------------ conversion table 
tCPCNV:		DB	00DH,020H	; CR to space
		DB	00AH,020H	; LF to space
		DB	'{', '^'	; { to ^
		DB	'}', '_'	; } to _
		DB	'~', '\'	; ~ to \

;================================================================= eoF CPCNV()

;===================================================================== CPMTR()
; Purpose.....: Turn printer motor on/off
; Input.......: HL': Address of I/O port table
;		CY:  1= turn on
;		     0= turn off
; Registers...: AF destroyed 
;=============================================================================
CPMTR:		PUSH	IX
		PUSH	BC

		LD	IX,(vCPPAT)	; IX= port table address
		LD	C,(IX+bCPPOM)	; C= motor port		
		LD	B,(IX+bCPPOM+1)	; B= motor bit value
		IN	A,(C)		; A= port contents
		JR	NC,CPMTRF	; CY=0, motor off
		
		OR	B		; turn on
		JR	CPMTRW		; go to write port

CPMTRF:		PUSH	AF		; save port value
		LD	A,B             ; invert motor value
		CPL
		LD	B,A
		POP	AF              ; recall port value
		AND	B               ; reset motor bit

CPMTRW:		OUT	(C),A		; write to motor port	

		POP	BC
		POP	IX
		RET
;===================================================================== CPMTR()				

;====================================================================== CPPR()
; Purpose.....: Print line buffer contents
;		The Christiani printer prints from right to left, the last
;		character first.
;		The buffer is searched for a CR terminator and the characters
;		until CR are counted. Unused space after CR will be printed
;		as spaces, as the printer always prints the whole line. If
;		no CR is found, the whole buffer is printed as it is.
; Input.......: IY: line buffer address, can also point to ROM
;		CY: 1= clear print buffer after printing
;		    0= do not touch print buffer (e.g. when printing static
;		       text from RAM)
; Output......: Z:  1= user key was pressed while printing
;		    0= key not pressed
; Registers...: AF destroyed
; Labels......: C)hristiani P)rinter P)R)int
;=============================================================================
CPPR:		PUSH	IY
		PUSH	IX
		PUSH	HL
		PUSH	DE
		PUSH	BC
		PUSH	AF		; save buffer clear flag
				
		LD	IX,vCPFLG	; ex240627-1: IX= flags address
		RES	iCPFUK,(IX)	; reset user key pressed flag
		
		PUSH	IY
		POP	HL		; HL= line buffer address

;----------------------------------- calc number of spaces at end of line (CR)
		PUSH	HL
		LD	B,bCPPBZ	; B= line size
		LD	C,0		; C= character counter
		LD	A,0DH		; search for CR
CPPRSL:		CP	(HL)		; is it CR?		
        	JR	Z,CPPRGO	; yes, go
     		INC	HL              ; bump buffer pointer
		INC	C               ; bump char count
		DJNZ	CPPRSL		; search whole buffer        	

CPPRGO:         LD	A,bCPPBZ	; A= line size
		SUB	C		; A= spaces at end of characters
		POP	HL		; HL= line buffer address

;----------------------------- make HL to point to line end (reverse printing)
		LD	BC,bCPPBZ	; BC= line size	
		ADD	HL,BC		; HL= end of line buffer
		DEC	HL
		LD	B,bCPPBZ	; B= character counter
		LD	E,A		; E= spaces to print first
		
;------------------------------------------------------------ printer motor on
		SCF
		CALL	CPMTR		; turn motor on

;---------------------------------------------------------- get port addresses
		LD	IY,(vCPPAT)	; IY= port table address

;------------------------------------------------------------- characters loop
		LD	C,(IY+bCPPOS)	; C= status port
		LD	D,(IY+bCPPOS+1)	; D= status value
				
;-------------------------------------------------------- wait until not ready
CPPRWR:		CALL	CPGUK		; check user key
		IN	A,(C)		; A= status port
		AND	D               ; check status bit
		JR	NZ,CPPRWR	; still ready, wait

;-------------------------------------------------------- print leading spaces
CPPRCL:		LD	A,E     	; A= spaces counter
		OR	A               ; more spaces to print?
		JR	Z,CPPRP		; no, print character
		DEC	E		; dec spaces counter
		LD	A,020H		;  
		JR	CPPRN		; print space

;-------------------------------------------------- write character to printer
CPPRP:		CALL	CPCNV		; character conversion
CPPRN:		DEC	HL		; HL points to previous
		AND	03FH		; we can only print 6 bits
		SUB	020H		; convert to binary
		XOR	020H		; bit5 is inverted (why?)
		OR	040H		; on PIO, keep motor on, does no harm
					; to PPI, PA6/D6, is not connected
		
		LD	C,(IY+bCPPOD)	; C= data port
		OUT	(C),A		; write to it
		LD	C,(IY+bCPPOS)	; C= status port

;------------------------------------ wait until character done, read user key
CPPRWD:		CALL	CPGUK		; check user key
		IN	A,(C)		
		AND	D
		JR	Z,CPPRWD        ; wait until character done

CPPRL: 		DJNZ	CPPRWR		; until whole buffer printed

;------------------------------------------------------------------- line done
		XOR	A
		CALL	CPMTR		; turn motor off		

		POP	AF		; recall clear buffer flag
		JR	NC,CPPRX	; CY=0, leave print buffer alone
		CALL	CPCB		; clear buffer for next line

CPPRX:		LD	A,(vCPFLG)	; A= flags
		CPL			; invert
		BIT	iCPFUK,A 	; ex240627-1: Z=1 if user key pressed

		POP	BC
		POP	DE
		POP	HL
		POP	IX
		POP	IY
		RET
;================================================================== eoF CPPR()

;====================================================================== CPPT()
; Purpose.....: Print zero-terminated text. Lines may be terminated with CR
;		Not used by Disassembler/Hexdump
; Input.......: DE: Address of text to print
; Output......:	Z:  1= user key was pressed
;		    0= it was not
; Registers...: AF destroyed
; Labels......: C)hristiani P)rinter P)rint T)ext
;=============================================================================
CPPT:		PUSH	IY
		PUSH	HL
		PUSH	DE
		PUSH	BC

		LD	IY,vCPPBF	; set print buffer for CPPR
			
CPPTLL:		LD	HL,vCPPBF	; HL= printer buffer
		LD	B,bCPPBZ        ; B= line length

CPPTCL:		LD	A,(DE)		; A= next character
		OR	A               ; terminator?
		JR	Z,CPPTR		; yes, print remainder
		LD	(HL),A		; no, write to print buffer
		INC	HL		; bump buffer pointer
		INC	DE              ; bump text pointer
		CP	0DH		; end of line?
		JR	Z,CPPTP		; yes, print line
		DJNZ	CPPTCL		; until one line done	

CPPTP:		SCF			; clear buffer after printing
		CALL	CPPR		; print the line		
		SCF
		CCF			; make sure CY=0
		JR	Z,CPPTX		; user key was pressed
		JR	CPPTLL		; go for next line

CPPTR:		LD	A,B             ; A= buffer counter
		CP	bCPPBZ          ; buffer empty?
		JR	Z,CPPTD         ; yes, skip printing
		SCF			; clear buffer after printing
		CALL	CPPR		; print last line (remainder)
CPPTD:		XOR	A		; normal exit, CY=0
		INC	A               ; and Z=0
		
CPPTX:		POP	BC
		POP	DE
		POP	HL
		POP	IY
		RET
;================================================================== eoF CPPT()

;====================================================================== CPLF()
; Purpose.....: Print line feed (empty line)
;		Not used by Disassembler/Hexdump
; Registers...: AF destroyed
; Labels......: C)hristiani P)rinter L)ine F)eed
;=============================================================================
CPLF:		PUSH	IY
		LD	IY,bCPLF	; IY= address of LF	
		XOR	A		; do not clear print buffer
		CALL	CPPR		; print empty line
		POP	IY
		RET

bCPLF:		DB	0DH
;================================================================== eoF CPLF()
		
;############################################################### EOF CPDRV.INC
		END

